//	Verify MIME content handling
//
// $Id: vMIMEContent.cc,v 2.3 1998/04/21 17:10:03 oleg Exp oleg $

#include <fstream.h>
#include "MIMEEntity.h"
#if defined(WIN32)
#include <strstrea.h>
#else
#include <strstream.h>
#endif

		// Sample multipart messages (from RFC 2046)
		// Note we now test the cases where headers could
		// be terminated with CR, LF, or CRLF
static const char Message_multipart_1_level [] =
	"From: Nathaniel Borenstein <nsb@bellcore.com>\n"
	"To: Ned Freed <ned@innosoft.com>\n"
	"Date: Mon, 22 Mar 1993 09:41:09 -0800 (PST)\r"
	"Subject: Formatted text mail\r\n"
	"MIME-Version: 1.0\r\n"
	"Content-Type: multipart/alternative; boundary=boundary42\r\n"
	"\r\n"
	"--boundary42\r\n"
	"Content-Type: text/plain; charset=us-ascii\r"
	"\r"
	"... plain text version of message goes here ...\r\n"
	"\r\n"
	"--boundary42\r\n"
	"Content-Type: text/enriched\n"
	"\n"
	"... RFC 1896 text/enriched version of same message\r\n"
	"\tgoes here ...\r\n"
	"\r\n"
	"--boundary42\r\n"
	"Content-Type: application/x-whatever\r\n"
	"\r\n"
	"... fanciest version of same message goes here ...\r\n"
	"\r\n"
	"--boundary42--\r\n";
	
static const char Message2_multipart_1_level [] =
	"From: Nathaniel Borenstein <nsb@bellcore.com>\r"
	"To: Ned Freed <ned@innosoft.com>\r"
     	"Date: Sun, 21 Mar 1993 23:56:48 -0800 (PST)\r"
     	"Subject: Sample message\r"
     	"MIME-Version: 1.0\r"
     	"Content-type: multipart/mixed; boundary=\"simple boundary\" \r"
	"\r"
     	"This is the preamble.  It is to be ignored, though it\r\n"
     	"is a handy place for composition agents to include an\r\n"
     	"explanatory note to non-MIME conformant readers.\r\n"
	"\r\n"
     	"--simple boundary\r\n"
     	"Content-type: text/plain; charset=us-ascii\n"
	"\r"
	"This is implicitly typed plain US-ASCII text.\r\n"
	"It does NOT end with a linebreak.\r\n"
	"--simple boundary\r\n"
	"Content-type: text/plain; charset=us-ascii\r"
	"\r"
     	"This is explicitly typed plain US-ASCII text.\r\n"
     	"It DOES end with a linebreak.\r\n"
	"\r\n"
	"--simple boundary--\r\n"
	"\r\n"
     	"This is the epilogue.  It is also to be ignored.";
	
static const char Message_multipart_2_level [] =
	"From: Moderator-Address\r\n"
	"To: Recipient-List\r\n"
	"Date: Mon, 22 Mar 1994 13:34:51 +0000\r\n"
	"Subject: Internet Digest, volume 42\r\n"
	"MIME-Version: 1.0\r\n"
	"Content-Type: multipart/mixed;\n"
	"\t boundary=\"---- main boundary ----\"\n"
	"\n"
	"------ main boundary ----\n"
	"Content-type: text/plain; charset=us-ascii\r\n"
	"\n"
       "...Introductory text or table of contents...\n"
	"\n"
	"------ main boundary ----\n"
	"Content-Type: multipart/digest;\r\n"
        " \t boundary=\"---- next message ----\" \r\n"
	"\r\n"
	"------ next message ----\n"
	"Content-type: text/plain; charset=us-ascii\r\n"
	"From: someone-else\r\n"
	"Date: Fri, 26 Mar 1993 11:13:32 +0200\r\n"
	"Subject: my opinion\r\n"
	"\r\n"
	"...body goes here ...\n"
	"\n"
	"------ next message ----\n"
	"Content-type: text/plain; charset=us-ascii\r\n"
	"From: someone-else-again\r\n"
	"Date: Fri, 26 Mar 1993 10:07:13 -0500\r\n"
	"Subject: my different opinion\r\n"
	"\r\n"
	"... another body goes here ...\n"
	"\n"
	"------ next message ------\n"
	"\n"
	"------ main boundary ------";

static void test_parmstring(const char sample_str [])
{
  cerr << "\nTesting parsing of a parameterized string "
       << '>' << sample_str << '<' << endl;
  StringCollection collection;
  StringCollection::Mark mark = collection.new_string();
  collection << sample_str << ends;
  ParameterizedToken parsed_str("__type-key",iststream(collection,mark));
  cerr << "Parsed string" << parsed_str;
}

			// An entity handler that simply dumps the entity
struct DumpEntity : public EntityHandler
{
  virtual bool operator () (const MIMEDiscreteEntity& entity);
};

bool DumpEntity::operator () (const MIMEDiscreteEntity& entity)
{
  cerr << "-->Loaded entity" << endl;
  cerr << entity.q_headers() << endl;
  cerr << "Content type: " << entity.q_content_type() << endl;
  cerr << "The body is loaded into the file " << 
  	entity.q_body_file_name() << endl;
  cerr << "which is\n" << ifstream(entity.q_body_file_name()).rdbuf()
       << "\n<<<" << endl;
  return true;
}

struct DumpCheckEntity : public DumpEntity
{
  const char * const anticipated_body;
  DumpCheckEntity(const char _anticipated_body [])
  	: anticipated_body(_anticipated_body) {}
  bool operator () (const MIMEDiscreteEntity& entity)
  {
    if( !DumpEntity::operator()(entity) )
      return false;
    strstream body;
    body << ifstream(entity.q_body_file_name()).rdbuf() << ends;
    if( strcmp(body.str(), anticipated_body) != 0 )
    {
      cerr << "Loaded entity's body differs from the expected\n"
           << anticipated_body << endl;
      return false;
    }
    return true;
  }
};

static void reset(strstream& stream)
{
  stream.clear();
#if !defined(WIN32)
  stream.freeze(0);
#else
  stream.rdbuf()->freeze(0);
#endif
  stream.seekp(0); stream.seekg(0);
}
			// Test loading single entities with a given body
static void check_single_entity(const char test_string [])
{
  cerr << "\nTest loading single entities with body >" << test_string
       << "<\n" << endl;
  strstream message_stream;
  
  {
    cerr << "Single discrete entity with no length given" << endl;
    reset(message_stream);
    message_stream << "From: me <me>\r\nContent-type: plain/text\r\n"
    	<< "\r\n"		// end-of-headers
    	<< test_string;
    assert( MIMEEntityStream::handle_entity(message_stream,
    	    DumpCheckEntity(test_string)) );
  }
  {
    cerr << "\nSingle discrete entity of specified length" << endl;
    char pad_char = 'X';
    reset(message_stream);
    message_stream << "From: me <me>\r\nContent-type: plain/text; area=1\r\n"
        << "Content-Length: " << strlen(test_string) << "\r\n"
    	<< "\r\n"		// end-of-headers
    	<< test_string << pad_char;
    assert( MIMEEntityStream::handle_entity(message_stream,
    	    DumpCheckEntity(test_string)) );
    assert( message_stream.get() == pad_char );
  }
  {
    cerr << "\nSingle discrete entity of no specified length"
         << "\npacked in a multipart message; no closing boundary" << endl;
    const char boundary [] = "simple-boundary";
    reset(message_stream);
    message_stream << "From: me <me>\r\n"
    		   << "Content-type: multipart/mixed; boundary="
    		   << boundary << "\r\n"
    		   << "\r\n"		// end-of-headers
    		   << "--" << boundary << "\r\n"
    		   << "Content-type: plain/text; area=\"1\"\r\n"
    		   << "X-one: 1\r\n"
    		   << "\r\n"		// end-of-headers
    		   << test_string;
    assert( MIMEEntityStream::handle_entity(message_stream,
    	    DumpCheckEntity(test_string)) );
  }
  {
    cerr << "\nSingle discrete entity of no specified length"
         << "\npacked in a multipart message " << endl;
    char pad_char = 'Y';
    const char boundary [] = "--42--";
    reset(message_stream);
    message_stream << "From: me <me>\r\n"
    		   << "Content-type: multipart/digest; boundary="
    		   << boundary << "\r\n"
    		   << "\r\n"		// end-of-headers
    		   << "--" << boundary << "  \t" << "\n"
    		   << "Content-type: plain/text; area=\"1\"; kind=3\r\n"
    		   << "X-one: 1\r\n"
    		   << "\r\n"		// end-of-headers
    		   << test_string 
    		   << "\n--" << boundary << "--\n" << pad_char;
    assert( MIMEEntityStream::handle_entity(message_stream,
    	    DumpCheckEntity(test_string)) );
    assert( message_stream.get() == '\n' );
    assert( message_stream.get() == pad_char );
  }
  {
    cerr << "\nSingle discrete entity of a specified length"
         << "\npacked in a multipart message " << endl;
    char pad_char = 'Z';
    const char boundary [] = "simple boundary";
    reset(message_stream);
    message_stream << "From: me <me>\r\n"
    		   << "Content-type: multipart/digest; boundary="
    		   << '"' << boundary << '"' << "\r\n"
    		   << "\r\n"		// end-of-headers
    		   << "preamble"
    		   << "\r\n--" << boundary << "  \t " << "\r"
    		   << "Content-type: plain/text; area=\"1\" \r\n"
    		   << "X-one:\t1\r\n"
        	   << "Content-Length: " << strlen(test_string) << "\r\n"
    		   << "\r\n"		// end-of-headers
    		   << test_string 
    		   << "\r\n--" << boundary << "--  \n" << pad_char;
    assert( MIMEEntityStream::handle_entity(message_stream,
    	    DumpCheckEntity(test_string)) );
    assert( message_stream.get() == '\n' );
    assert( message_stream.get() == pad_char );
  }
  {
    cerr << "\nSingle discrete entity of a specified length"
         << "\npacked in a multipart message with no end terminator" << endl;
    const char boundary [] = "simple boundary ";
    strstream message_stream;
    message_stream << "From: me <me>\r\n"
    		   << "Content-type: multipart/digest; boundary="
    		   << '"' << boundary << '"' << "\n"
    		   << "\n"		// end-of-headers
    		   << " preamble "
    		   << "\r\n--" << boundary << "  \t " << "\r"
    		   << "Content-type: plain/text; area=\"1\" \r"
    		   << "X-one:\t1\r"
        	   << "Content-Length: " << strlen(test_string) << "\r"
    		   << "\r"		// end-of-headers
    		   << test_string;
    assert( MIMEEntityStream::handle_entity(message_stream,
    	    DumpCheckEntity(test_string)) );
    assert( message_stream.eof() );
  }
}

static void check_multipart(const char message_str [])
{
  cerr << "\n<<<Checking a multipart message from a given string\n" << endl;
  strstream message_stream; message_stream << message_str;
  assert( MIMEEntityStream::handle_entity(message_stream,
  	  DumpEntity()) );
  cerr << "\n>>>Done checking multipart" << endl;
}

main(void)
{

  test_parmstring("text/html");
  test_parmstring("text/html; AREA=123");
  test_parmstring("text/html; AREA=\"123\"");
  test_parmstring("text/html; AREA=\"123\"; KINd=1");

  check_single_entity("123\n45");
  check_multipart(Message_multipart_1_level);
  check_multipart(Message2_multipart_1_level);
  check_multipart(Message_multipart_2_level);
  cerr << "\nAll tests passed" << endl;
}
     
