/*
 ************************************************************************
 *
 *		     Validate a RFC822 pseudo-filesystem
 * which provides a file-centric view to a structured document (an
 * RFC822-message)
 *
 * $Id: vhttp822.c,v 1.1 1999/05/14 23:31:39 oleg Exp oleg $
 *
 ************************************************************************
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <assert.h>
#if defined(ACCESSPERMS)
#define S_IAMB ACCESSPERMS
#endif

                                // Make sure that a system call went well
                                // If a system call (say, fcntl()) fails
                                // it returns -1 and sets the errno
#define do_well(FX) (void)( (FX) < 0 ? (perror("System error"), \
        fprintf(stderr,"Failed system call " #FX " at line %d of `%s'.\n", \
               __LINE__, __FILE__), exit(4)) : 1)

			/* Get a stat of a "file"/"directory" and print it out */
static void show_stat(const char filename [])
{
  struct stat buf;
  memset(&buf,0,sizeof(buf));
  do_well( lstat(filename,&buf) );
  printf("\nInformation for a file '%s'...",filename);
  printf("\n\tThe file is a %s, permission flags %03o",
	 S_ISREG(buf.st_mode) ? "regular file" :
	 S_ISDIR(buf.st_mode) ? "directory" :
	 S_ISLNK(buf.st_mode) ? "symbolic link" :
	 S_ISFIFO(buf.st_mode) ? "FIFO pipe" :
	 "something else",
	 S_IAMB & buf.st_mode);
  printf("\n\tdevno:ino\t%d:%d\tnlinks\t%d",buf.st_dev, buf.st_ino, buf.st_nlink);
  printf("\n\tuid:gid  \t%d:%d",buf.st_uid, buf.st_gid);
  printf("\n\tsize     \t%d\tblocks\t%d",buf.st_size, buf.st_blocks);
  printf("\n\tLast access time:\t%s",ctime(&(buf.st_atime)));
  printf("\tLast modif time:   \t%s",ctime(&(buf.st_mtime)));
  printf("\tinode time:        \t%s",ctime(&(buf.st_ctime)));
}

		/* Print out the file */
static void show_file(const char filename [])
{
  struct stat buf; register int i; register int c;
  FILE * fp;
  memset(&buf,0,sizeof(buf));
  do_well( stat(filename,&buf) );
  assert( S_ISREG(buf.st_mode) );
  printf("\n\nA %d-byte long file '%s' is as follows\n>>>",buf.st_size,filename);
  assert( (fp = fopen(filename,"rb")) != (FILE*)0 );
  for(i=0; (c=getc(fp)) != EOF; i++)
    putchar(c);
  assert( i == buf.st_size );
  fclose(fp);
  printf("\n<<<\n\n");
}


int main(void)
{
  printf("\nValidating an RFC822 pseudo-filesystem\n");
  show_stat("http://your.host.name/cgi-bin/admin/HFS822-server.pl/DeepestRoot/tmp/message1");
  show_file("http://your.host.name/cgi-bin/admin/HFS822-server.pl/DeepestRoot/tmp/message1");
  show_stat("http://your.host.name/cgi-bin/admin/HFS822-server.pl/DeepestRoot/tmp/message1/");
  show_stat("http://your.host.name/cgi-bin/admin/HFS822-server.pl/DeepestRoot/tmp/message1/.");
  show_stat("http://your.host.name/cgi-bin/admin/HFS822-server.pl/DeepestRoot/tmp/message1/date");
  show_file("http://your.host.name/cgi-bin/admin/HFS822-server.pl/DeepestRoot/tmp/message1/Date");
  show_file("http://your.host.name/cgi-bin/admin/HFS822-server.pl/DeepestRoot/tmp/message1/Subject");
  show_file("http://your.host.name/cgi-bin/admin/HFS822-server.pl/DeepestRoot/tmp/message1/body");
  return 0;
}
